// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDEGAMES_VERSION_H
#define KDEGAMES_VERSION_H

#define KDEGAMES_VERSION_STRING "6.0.260400"
#define KDEGAMES_VERSION_MAJOR 6
#define KDEGAMES_VERSION_MINOR 0
#define KDEGAMES_VERSION_PATCH 260400
#define KDEGAMES_VERSION ((6<<16)|(0<<8)|(260400))

#endif
